@@ -30,7 +30,7 @@ const MIN: Decimal = Decimal {
     hi: 4_294_967_295,
 };
 
-/// The smallest value that can be represented by this decimal type.
+/// The largest value that can be represented by this decimal type.
 const MAX: Decimal = Decimal {
     flags: 0,
     lo: 4_294_967_295,
@@ -38,21 +38,48 @@ const MAX: Decimal = Decimal {
     hi: 4_294_967_295,
 };
 
-/// A constant representing 0.
 const ZERO: Decimal = Decimal {
     flags: 0,
     lo: 0,
     mid: 0,
     hi: 0,
 };
-
-/// A constant representing 1.
 const ONE: Decimal = Decimal {
     flags: 0,
     lo: 1,
     mid: 0,
     hi: 0,
 };
+const TWO: Decimal = Decimal {
+    flags: 0,
+    lo: 2,
+    mid: 0,
+    hi: 0,
+};
+const TEN: Decimal = Decimal {
+    flags: 0,
+    lo: 10,
+    mid: 0,
+    hi: 0,
+};
+const ONE_HUNDRED: Decimal = Decimal {
+    flags: 0,
+    lo: 100,
+    mid: 0,
+    hi: 0,
+};
+const ONE_THOUSAND: Decimal = Decimal {
+    flags: 0,
+    lo: 1000,
+    mid: 0,
+    hi: 0,
+};
+const NEGATIVE_ONE: Decimal = Decimal {
+    flags: 2147483648,
+    lo: 1,
+    mid: 0,
+    hi: 0,
+};
 
 /// `UnpackedDecimal` contains unpacked representation of `Decimal` where each component
 /// of decimal-format stored in it's own field
@@ -138,6 +165,65 @@ impl Decimal {
     pub const ZERO: Decimal = ZERO;
     /// A constant representing 1.
     pub const ONE: Decimal = ONE;
+    /// A constant representing -1.
+    pub const NEGATIVE_ONE: Decimal = NEGATIVE_ONE;
+    /// A constant representing 2.
+    pub const TWO: Decimal = TWO;
+    /// A constant representing 10.
+    pub const TEN: Decimal = TEN;
+    /// A constant representing 100.
+    pub const ONE_HUNDRED: Decimal = ONE_HUNDRED;
+    /// A constant representing 1000.
+    pub const ONE_THOUSAND: Decimal = ONE_THOUSAND;
+
+    /// A constant representing π as 3.1415926535897932384626433833
+    #[cfg(feature = "maths")]
+    pub const PI: Decimal = Decimal {
+        flags: 1835008,
+        lo: 1102470953,
+        mid: 185874565,
+        hi: 1703060790,
+    };
+    /// A constant representing π/2 as 1.5707963267948966192313216916
+    #[cfg(feature = "maths")]
+    pub const HALF_PI: Decimal = Decimal {
+        flags: 1835008,
+        lo: 2698719124,
+        mid: 92937282,
+        hi: 851530395,
+    };
+    /// A constant representing π/4 as 0.7853981633974483096156608458
+    #[cfg(feature = "maths")]
+    pub const QUARTER_PI: Decimal = Decimal {
+        flags: 1835008,
+        lo: 1349359562,
+        mid: 2193952289,
+        hi: 425765197,
+    };
+    /// A constant representing 2π as 6.2831853071795864769252867666
+    #[cfg(feature = "maths")]
+    pub const TWO_PI: Decimal = Decimal {
+        flags: 1835008,
+        lo: 2204941906,
+        mid: 371749130,
+        hi: 3406121580,
+    };
+    /// A constant representing Euler's number (e) as 2.7182818284590452353602874714
+    #[cfg(feature = "maths")]
+    pub const E: Decimal = Decimal {
+        flags: 1835008,
+        lo: 2239425882,
+        mid: 3958169141,
+        hi: 1473583531,
+    };
+    /// A constant representing the inverse of Euler's number (1/e) as 0.3678794411714423215955237702
+    #[cfg(feature = "maths")]
+    pub const E_INVERSE: Decimal = Decimal {
+        flags: 1835008,
+        lo: 2384059206,
+        mid: 2857938002,
+        hi: 199427844,
+    };
 
     /// Returns a `Decimal` with a 64 bit `m` representation and corresponding `e` scale.
     ///
@@ -160,28 +246,42 @@ impl Decimal {
     /// ```
     #[must_use]
     pub fn new(num: i64, scale: u32) -> Decimal {
+        match Self::try_new(num, scale) {
+            Err(e) => panic!("{}", e),
+            Ok(d) => d,
+        }
+    }
+
+    /// Checked version of `Decimal::new`. Will return `Err` instead of panicking at run-time.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// use rust_decimal::Decimal;
+    ///
+    /// let max = Decimal::try_new(i64::MAX, u32::MAX);
+    /// assert!(max.is_err());
+    /// ```
+    pub const fn try_new(num: i64, scale: u32) -> crate::Result<Decimal> {
         if scale > MAX_PRECISION {
-            panic!(
-                "Scale exceeds the maximum precision allowed: {} > {}",
-                scale, MAX_PRECISION
-            );
+            return Err(Error::ScaleExceedsMaximumPrecision(scale));
         }
         let flags: u32 = scale << SCALE_SHIFT;
         if num < 0 {
             let pos_num = num.wrapping_neg() as u64;
-            return Decimal {
+            return Ok(Decimal {
                 flags: flags | SIGN_MASK,
                 hi: 0,
                 lo: (pos_num & U32_MASK) as u32,
                 mid: ((pos_num >> 32) & U32_MASK) as u32,
-            };
+            });
         }
-        Decimal {
+        Ok(Decimal {
             flags,
             hi: 0,
             lo: (num as u64 & U32_MASK) as u32,
             mid: ((num as u64 >> 32) & U32_MASK) as u32,
-        }
+        })
     }
 
     /// Creates a `Decimal` using a 128 bit signed `m` representation and corresponding `e` scale.
@@ -211,7 +311,7 @@ impl Decimal {
         }
     }
 
-    /// Checked version of `from_i128_with_scale`. Will return `Err` instead
+    /// Checked version of `Decimal::from_i128_with_scale`. Will return `Err` instead
     /// of panicking at run-time.
     ///
     /// # Example
@@ -222,7 +322,7 @@ impl Decimal {
     /// let max = Decimal::try_from_i128_with_scale(i128::MAX, u32::MAX);
     /// assert!(max.is_err());
     /// ```
-    pub fn try_from_i128_with_scale(num: i128, scale: u32) -> crate::Result<Decimal> {
+    pub const fn try_from_i128_with_scale(num: i128, scale: u32) -> crate::Result<Decimal> {
         if scale > MAX_PRECISION {
             return Err(Error::ScaleExceedsMaximumPrecision(scale));
         }
@@ -297,7 +397,7 @@ impl Decimal {
                 flags: flags & SCALE_MASK,
             }
         } else {
-            Decimal { lo, mid, hi, flags }
+            Decimal { flags, hi, lo, mid }
         }
     }
 
@@ -437,7 +537,7 @@ impl Decimal {
     /// ```
     /// use rust_decimal::Decimal;
     ///
-    /// let mut one = Decimal::new(1, 0);
+    /// let mut one = Decimal::ONE;
     /// one.set_sign(false);
     /// assert_eq!(one.to_string(), "-1");
     /// ```
@@ -457,7 +557,7 @@ impl Decimal {
     /// ```
     /// use rust_decimal::Decimal;
     ///
-    /// let mut one = Decimal::new(1, 0);
+    /// let mut one = Decimal::ONE;
     /// one.set_sign_positive(false);
     /// assert_eq!(one.to_string(), "-1");
     /// ```
@@ -481,7 +581,7 @@ impl Decimal {
     /// ```
     /// use rust_decimal::Decimal;
     ///
-    /// let mut one = Decimal::new(1, 0);
+    /// let mut one = Decimal::ONE;
     /// one.set_sign_negative(true);
     /// assert_eq!(one.to_string(), "-1");
     /// ```
@@ -501,7 +601,7 @@ impl Decimal {
     /// ```
     /// use rust_decimal::Decimal;
     ///
-    /// let mut one = Decimal::new(1, 0);
+    /// let mut one = Decimal::ONE;
     /// one.set_scale(5).unwrap();
     /// assert_eq!(one.to_string(), "0.00001");
     /// ```
@@ -528,14 +628,21 @@ impl Decimal {
     /// # Example
     ///
     /// ```
-    /// use rust_decimal::Decimal;
+    /// use rust_decimal::prelude::*;
     ///
-    /// let mut number = Decimal::new(1_123, 3);
+    /// // Rescaling to a higher scale preserves the value
+    /// let mut number = Decimal::from_str("1.123").unwrap();
+    /// assert_eq!(number.scale(), 3);
     /// number.rescale(6);
-    /// assert_eq!(number, Decimal::new(1_123_000, 6));
-    /// let mut round = Decimal::new(145, 2);
-    /// round.rescale(1);
-    /// assert_eq!(round, Decimal::new(15, 1));
+    /// assert_eq!(number.to_string(), "1.123000");
+    /// assert_eq!(number.scale(), 6);
+    ///
+    /// // Rescaling to a lower scale forces the number to be rounded
+    /// let mut number = Decimal::from_str("1.45").unwrap();
+    /// assert_eq!(number.scale(), 2);
+    /// number.rescale(1);
+    /// assert_eq!(number.to_string(), "1.5");
+    /// assert_eq!(number.scale(), 1);
     /// ```
     pub fn rescale(&mut self, scale: u32) {
         let mut array = [self.lo, self.mid, self.hi];
@@ -1050,6 +1157,164 @@ impl Decimal {
         self.round_dp_with_strategy(dp, RoundingStrategy::MidpointNearestEven)
     }
 
+    /// Returns `Some(Decimal)` number rounded to the specified number of significant digits. If
+    /// the resulting number is unable to be represented by the `Decimal` number then `None` will
+    /// be returned.  
+    /// When the number of significant figures of the `Decimal` being rounded is greater than the requested
+    /// number of significant digits then rounding will be performed using `MidpointNearestEven` strategy.
+    ///
+    /// # Arguments
+    /// * `digits`: the number of significant digits to round to.
+    ///
+    /// # Remarks
+    /// A significant figure is determined using the following rules:
+    /// 1. Non-zero digits are always significant.
+    /// 2. Zeros between non-zero digits are always significant.
+    /// 3. Leading zeros are never significant.
+    /// 4. Trailing zeros are only significant if the number contains a decimal point.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use rust_decimal::Decimal;
+    /// use core::str::FromStr;
+    ///
+    /// let value = Decimal::from_str("305.459").unwrap();
+    /// assert_eq!(value.round_sf(0), Some(Decimal::from_str("0").unwrap()));
+    /// assert_eq!(value.round_sf(1), Some(Decimal::from_str("300").unwrap()));
+    /// assert_eq!(value.round_sf(2), Some(Decimal::from_str("310").unwrap()));
+    /// assert_eq!(value.round_sf(3), Some(Decimal::from_str("305").unwrap()));
+    /// assert_eq!(value.round_sf(4), Some(Decimal::from_str("305.5").unwrap()));
+    /// assert_eq!(value.round_sf(5), Some(Decimal::from_str("305.46").unwrap()));
+    /// assert_eq!(value.round_sf(6), Some(Decimal::from_str("305.459").unwrap()));
+    /// assert_eq!(value.round_sf(7), Some(Decimal::from_str("305.4590").unwrap()));
+    /// assert_eq!(Decimal::MAX.round_sf(1), None);
+    /// ```
+    #[must_use]
+    pub fn round_sf(&self, digits: u32) -> Option<Decimal> {
+        self.round_sf_with_strategy(digits, RoundingStrategy::MidpointNearestEven)
+    }
+
+    /// Returns `Some(Decimal)` number rounded to the specified number of significant digits. If
+    /// the resulting number is unable to be represented by the `Decimal` number then `None` will
+    /// be returned.  
+    /// When the number of significant figures of the `Decimal` being rounded is greater than the requested
+    /// number of significant digits then rounding will be performed using the provided [RoundingStrategy].
+    ///
+    /// # Arguments
+    /// * `digits`: the number of significant digits to round to.
+    /// * `strategy`: if required, the rounding strategy to use.
+    ///
+    /// # Remarks
+    /// A significant figure is determined using the following rules:
+    /// 1. Non-zero digits are always significant.
+    /// 2. Zeros between non-zero digits are always significant.
+    /// 3. Leading zeros are never significant.
+    /// 4. Trailing zeros are only significant if the number contains a decimal point.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use rust_decimal::{Decimal, RoundingStrategy};
+    /// use core::str::FromStr;
+    ///
+    /// let value = Decimal::from_str("305.459").unwrap();
+    /// assert_eq!(value.round_sf_with_strategy(0, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("0").unwrap());
+    /// assert_eq!(value.round_sf_with_strategy(1, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("300").unwrap());
+    /// assert_eq!(value.round_sf_with_strategy(2, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("300").unwrap());
+    /// assert_eq!(value.round_sf_with_strategy(3, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("305").unwrap());
+    /// assert_eq!(value.round_sf_with_strategy(4, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("305.4").unwrap());
+    /// assert_eq!(value.round_sf_with_strategy(5, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("305.45").unwrap());
+    /// assert_eq!(value.round_sf_with_strategy(6, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("305.459").unwrap());
+    /// assert_eq!(value.round_sf_with_strategy(7, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("305.4590").unwrap());
+    /// assert_eq!(Decimal::MAX.round_sf_with_strategy(1, RoundingStrategy::ToZero).unwrap(), Decimal::from_str("70000000000000000000000000000").unwrap());
+    /// ```
+    #[must_use]
+    pub fn round_sf_with_strategy(&self, digits: u32, strategy: RoundingStrategy) -> Option<Decimal> {
+        if self.is_zero() || digits == 0 {
+            return Some(Decimal::ZERO);
+        }
+
+        // We start by grabbing the mantissa and figuring out how many significant figures it is
+        // made up of. We do this by just dividing by 10 and checking remainders - effectively
+        // we're performing a naive log10.
+        let mut working = self.mantissa_array3();
+        let mut mantissa_sf = 0;
+        while !ops::array::is_all_zero(&working) {
+            let _remainder = ops::array::div_by_u32(&mut working, 10u32);
+            mantissa_sf += 1;
+            if working[2] == 0 && working[1] == 0 && working[0] == 1 {
+                break;
+            }
+        }
+
+        // Total significant figures in the full number also needs to include scale.
+        // e.g. mantissa 1, scale of 6 = 0.000001 (6 sf), mantissa 1234, scale of 3 = 1.234 (4 sf)
+        let scale = self.scale();
+        let current_sf = if scale > mantissa_sf { scale } else { mantissa_sf };
+
+        match digits.cmp(&current_sf) {
+            Ordering::Greater => {
+                // If we're requesting a higher number of significant figures, we rescale
+                let mut array = [self.lo, self.mid, self.hi];
+                let mut value_scale = self.scale();
+                ops::array::rescale_internal(&mut array, &mut value_scale, scale + digits - current_sf);
+                Some(Decimal {
+                    lo: array[0],
+                    mid: array[1],
+                    hi: array[2],
+                    flags: flags(self.is_sign_negative(), value_scale),
+                })
+            }
+            Ordering::Less => {
+                // We're requesting a lower number of significant digits.
+                let diff = current_sf - digits;
+                // If the diff is greater than the scale we're focused on the integral. Otherwise, we can
+                // just round.
+                if diff > scale {
+                    use crate::constants::BIG_POWERS_10;
+                    // We need to adjust the integral portion. This also should be rounded, consequently
+                    // we reduce the number down, round it, and then scale back up.
+                    // E.g. If we have 305.459 scaling to a sf of 2. We first, reduce the number
+                    // down to 30.5459, round it to 31 and then scale it back up to 310.
+                    let mut num = *self;
+                    let mut exp = (diff - scale) as usize;
+                    while exp > 0 {
+                        let pow;
+                        if exp >= BIG_POWERS_10.len() {
+                            pow = Decimal::from(BIG_POWERS_10[BIG_POWERS_10.len() - 1]);
+                            exp -= BIG_POWERS_10.len();
+                        } else {
+                            pow = Decimal::from(BIG_POWERS_10[exp - 1]);
+                            exp = 0;
+                        }
+                        num = num.checked_div(pow)?;
+                    }
+                    let mut num = num.round_dp_with_strategy(0, strategy).trunc();
+                    let mut exp = (current_sf - digits - scale) as usize;
+                    while exp > 0 {
+                        let pow;
+                        if exp >= BIG_POWERS_10.len() {
+                            pow = Decimal::from(BIG_POWERS_10[BIG_POWERS_10.len() - 1]);
+                            exp -= BIG_POWERS_10.len();
+                        } else {
+                            pow = Decimal::from(BIG_POWERS_10[exp - 1]);
+                            exp = 0;
+                        }
+                        num = num.checked_mul(pow)?;
+                    }
+                    Some(num)
+                } else {
+                    Some(self.round_dp_with_strategy(scale - diff, strategy))
+                }
+            }
+            Ordering::Equal => {
+                // Case where significant figures = requested significant digits.
+                Some(*self)
+            }
+        }
+    }
+
     /// Convert `Decimal` to an internal representation of the underlying struct. This is useful
     /// for debugging the internal state of the object.
     ///
@@ -1735,10 +2000,7 @@ impl ToPrimitive for Decimal {
     fn to_f64(&self) -> Option<f64> {
         if self.scale() == 0 {
             let integer = self.to_i64();
-            match integer {
-                Some(i) => Some(i as f64),
-                None => None,
-            }
+            integer.map(|i| i as f64)
         } else {
             let sign: f64 = if self.is_sign_negative() { -1.0 } else { 1.0 };
             let mut mantissa: u128 = self.lo.into();
@@ -1752,7 +2014,7 @@ impl ToPrimitive for Decimal {
             let frac_f64 = (frac_part as f64) / (precision as f64);
             let value = sign * ((integral_part as f64) + frac_f64);
             let round_to = 10f64.powi(self.scale() as i32);
-            Some(value * round_to / round_to)
+            Some((value * round_to).round() / round_to)
         }
     }
 }
@@ -1844,7 +2106,7 @@ impl<'a, 'b> Add<&'b Decimal> for &'a Decimal {
 
     #[inline(always)]
     fn add(self, other: &Decimal) -> Decimal {
-        match ops::add_impl(&self, other) {
+        match ops::add_impl(self, other) {
             CalculationResult::Ok(sum) => sum,
             _ => panic!("Addition overflowed"),
         }
@@ -1886,7 +2148,7 @@ impl<'a, 'b> Sub<&'b Decimal> for &'a Decimal {
 
     #[inline(always)]
     fn sub(self, other: &Decimal) -> Decimal {
-        match ops::sub_impl(&self, other) {
+        match ops::sub_impl(self, other) {
             CalculationResult::Ok(sum) => sum,
             _ => panic!("Subtraction overflowed"),
         }
@@ -1928,7 +2190,7 @@ impl<'a, 'b> Mul<&'b Decimal> for &'a Decimal {
 
     #[inline]
     fn mul(self, other: &Decimal) -> Decimal {
-        match ops::mul_impl(&self, other) {
+        match ops::mul_impl(self, other) {
             CalculationResult::Ok(prod) => prod,
             _ => panic!("Multiplication overflowed"),
         }
@@ -1969,7 +2231,7 @@ impl<'a, 'b> Div<&'b Decimal> for &'a Decimal {
     type Output = Decimal;
 
     fn div(self, other: &Decimal) -> Decimal {
-        match ops::div_impl(&self, other) {
+        match ops::div_impl(self, other) {
             CalculationResult::Ok(quot) => quot,
             CalculationResult::Overflow => panic!("Division overflowed"),
             CalculationResult::DivByZero => panic!("Division by zero"),
@@ -2012,7 +2274,7 @@ impl<'a, 'b> Rem<&'b Decimal> for &'a Decimal {
 
     #[inline]
     fn rem(self, other: &Decimal) -> Decimal {
-        match ops::rem_impl(&self, other) {
+        match ops::rem_impl(self, other) {
             CalculationResult::Ok(rem) => rem,
             CalculationResult::Overflow => panic!("Division overflowed"),
             CalculationResult::DivByZero => panic!("Division by zero"),
